package guru.kake.ronove

import javafx.beans.property.SimpleStringProperty
import javafx.scene.control.*
import javafx.scene.paint.Color
import tornadofx.*

/**
 * Window for selecting MTL service provider and setting its API key
 */
class TranslationSettings : View("Settings") {
    private lateinit var applyButton : Button
    val toggleGroup = ToggleGroup()
    private var backends: MutableList<MTLSettingsMenu<TranslationService>> = mutableListOf()
    override val root = form {
        label {
            text = "Be aware that use of translation APIs may incur costs in accordance to their usage terms. Check with " +
                    "the translation service provider for their rates. While Ronove will attempt to minimize API calls " +
                    "where appropriate, be aware that for some providers even testing the API key may be a billed operation."
            isWrapText = true
            prefWidth = 300.0
        }

        fun MTLSettingsMenu<*>.add() {
            @Suppress("UNCHECKED_CAST")
            backends.add(this as MTLSettingsMenu<TranslationService>)
            this@form += this
        }

        MTLSettingsMenu(this@TranslationSettings, "DeepL") { DeepLTranslator(it) }.add()
        MTLSettingsMenu(this@TranslationSettings, "Google Translate API v2") { GoogleTranslator(it) }.add()

        hbox {
            applyButton = button("Apply") {
                isDefaultButton = true
                isDisable = true
                action {
                    val backend = backends.find {it.radio == toggleGroup.selectedToggle}
                    TranslationBackend.setService(backend!!.service!!, backend.name)
                    close()
                }
            }
            button("Cancel") {
                isCancelButton = true
                action {
                    close()
                }
            }
        }
    }

    /**
     * Validates the settings and enables the Apply button accordingly
     */
    fun updateButton() {
        applyButton.isDisable = backends.find { it.radio == toggleGroup.selectedToggle }?.service == null
        if (!applyButton.isDisable) { applyButton.requestFocus() }
    }
}

/**
 * UI Fragment to implement a settings menu for a translation backend requiring one API key as string
 */
private class MTLSettingsMenu<T: TranslationService>(private val parent: TranslationSettings, val name: String, val constructor: (String) -> T)
    : Fragment(name) {
    private lateinit var statusLabel: Label
    private lateinit var keyField: PasswordField
    private lateinit var saveButton: Button
    lateinit var radio: RadioButton
    var service: T? = null
    private val key = SimpleStringProperty()

    override val root = fieldset {
        radio = radiobutton(name, this@MTLSettingsMenu.parent.toggleGroup) {
            action { this@MTLSettingsMenu.parent.updateButton() }
        }
        field("API key") {
            keyField = passwordfield(key) {
                action { saveButtonAction() }
            }
        }
        saveButton = button("Save & test") {
            action { saveButtonAction() }
        }
        statusLabel = label()
    }

    init {
        key.value = ""
        if (TranslationBackend.serviceName.value == name) {
            key.value = "**********"
            radio.isSelected = true
        }
        saveButton.isDisable = true
        key.onChange { saveButton.isDisable = key.value == "" || key.value == "**********" }
    }

    private fun saveButtonAction() {
        try {
            service = constructor(keyField.text)
            statusLabel.text = "API initialized succesfully!"
            statusLabel.textFill = Color.GREEN
        } catch (e: TranslationServiceException) {
            statusLabel.text = e.message
            statusLabel.textFill = Color.RED
        } finally {
            parent.updateButton()
        }
    }
}